import React from 'react';
import PropTypes from 'prop-types';

import { Stepper } from './Stepper';
import { BreadcrumbContainer } from './styles';

/**
 * Breacrumb stepper component: 
 * This is a stepper component in a breacrumb style. 
 * You can use it by passing all the possible steps and a selected prop
 * to tell the component which step is selected, then the component is going to
 * render the step component you passed as a page.
 * You also can pass a onClick prop to control what happens when you click on
 * a step
**/
export function BreadcrumbStepper({
  selected,
  steps,
  onClick
}) {
  const { component: Component } = steps[selected];

  return (
    <BreadcrumbContainer>
      <Stepper selected={selected} steps={steps} onClick={onClick} />

      <br />

      <Component />
    </BreadcrumbContainer>
  )
}

BreadcrumbStepper.propTypes = {
  /**
   * The index of the selected step in the steps array (0 based numbering)
   */
  selected: PropTypes.number.isRequired,
  /**
   * An array containing all the steps to be rendered. Must follow:
   * [
   *  { title: 'Step 1', component: Step1Component },
   *  { title: 'Step 2', component: Step2Component },
   *  { title: 'Step 3', component: Step3Component },
   * ]
   */
  steps: PropTypes.arrayOf(PropTypes.shape({
    title: PropTypes.string.isRequired,
    component: PropTypes.oneOfType([
      PropTypes.element,
      PropTypes.func,
    ]).isRequired,
  })).isRequired,
  /**
   * A function that will be called when the user clicks on a step.
   * It must receive 1 step param.
   */
  onClick: PropTypes.func,
};

BreadcrumbStepper.defaultProps = {
  onClick: null,
};
