import React, { useMemo } from "react";
import PropTypes from "prop-types";

import { Button as MDButton } from "@material-ui/core";

import { colors, typography } from "../../shared/styles";

/**
- Default button component
**/
export function Button({
  children,
  backgroundColor,
  color,
  outlined,
  isStartIcon,
  icon,
  size,
  onClick,
  disabled,
  fullWidth,
  ...props
}) {
  const bgColor = useMemo(() => {
    if (disabled) return colors.lightest;

    // if (backgroundColor) return backgroundColor;

    if (outlined) return colors.lightest;

    return backgroundColor || "#9CD186";
  }, [backgroundColor, disabled, outlined]);

  const textColor = useMemo(() => {
    if (disabled) return colors.medium;

    if (color) return color;

    if (outlined) return backgroundColor || "#9CD186";

    return colors.lightest;
  }, [backgroundColor, color, disabled, outlined]);

  const borderColor = useMemo(() => {
    if (disabled) return colors.medium;

    // if (backgroundColor) return backgroundColor;

    return backgroundColor || "#9CD186";
  }, [backgroundColor, disabled]);

  const fontWeight = useMemo(() => {
    if (disabled) return typography.weight.regular;

    if (outlined) return typography.weight.bold;

    return typography.weight.regular;
  }, [disabled, outlined]);

  return (
    <MDButton
      fullWidth={fullWidth}
      size={size}
      variant="outlined"
      startIcon={isStartIcon && icon}
      endIcon={!isStartIcon && icon}
      style={{
        width: fullWidth && "100%",
        textTransform: "none",
        backgroundColor: bgColor,
        color: textColor,
        fontFamily: typography.type.secondary,
        fontSize: typography.size.s3,
        fontWeight: fontWeight,
        borderWidth: "2px",
        borderColor: borderColor,
        borderRadius: "8px",
      }}
      onClick={onClick}
      disabled={disabled}
      {...props}
    >
      {children}
    </MDButton>
  );
}

Button.propTypes = {
  /**
   * The content that will be shown inside the button
   */
  children: PropTypes.any,
  /**
   * A background color for the button.
   * Default: #85B074
   */
  backgroundColor: PropTypes.string,
  /**
   * A color for the button text.
   * Default: #fff
   */
  color: PropTypes.string,
  /**
   * If true, icon will be displayed at the start of the button.
   * Default: false
   */
  isStartIcon: PropTypes.bool,
  /**
   * The icon to be shown
   */
  icon: PropTypes.any,
  /**
   * The size of the button.
   * Accepted sizes: 'large', 'medium', 'small'
   */
  size: PropTypes.oneOf(["large", "medium", "small"]),
  /**
   * Button click function
   */
  onClick: PropTypes.func,
  /**
   * Button outlined style
   */
  outlined: PropTypes.bool,
  /**
   * Disable the button
   */
  disabled: PropTypes.bool,
  /**
   * If true, the button will take up the full width of its container
   */
  fullWidth: PropTypes.bool,
};

Button.defaultProps = {
  children: "Send",
  backgroundColor: null,
  color: null,
  isStartIcon: false,
  icon: false,
  size: "large",
  onClick: null,
  outlined: false,
  disabled: false,
  fullWidth: false,
};
