import React, { useState } from "react";
import PropTypes from "prop-types";
import styled from "styled-components";
import {
  Drawer as MuiDrawer,
  makeStyles,
  List,
  ListItem,
  ListItemText,
  IconButton,
  Avatar,
} from "@material-ui/core";
import { MdMenu } from "react-icons/md";
import { List as ListIcon } from "@material-ui/icons";
import { typography } from "../../shared/styles";
import {
  ListMenuItemStyled,
  ListItemIconStyled,
  TypographyMenuItemStyled,
} from "./styles";

const useStyles = makeStyles({
  drawer: {
    width: "20%",
  },
  drawerPaper: {
    // borderTopRightRadius: "10px",
    // borderBottomRightRadius: "10px",
    // borderColor: colors.primaryDark,
    backgroundColor: "#F8F8F8",
    boxShadow: "0px 4px 4px rgba(0, 0, 0, 0.25)",
    // backgroundColor: colors.primaryDark,
  },
});

const StyledMuiDrawer = styled(MuiDrawer)`
  .MuiDrawer-paper {
    width: 20%;
    padding-top: 110px;
  }
  @media (max-width: 800px) {
    .MuiDrawer-paper {
      width: 40%;
    }
  }
  // in case of print dont show menu
  @media print {
    .MuiDrawer-paper {
    display: none;
    visibility: hidden;
  }
}
`;

const StyledIconButton = styled(IconButton)`
  display: none;
  @media (max-width: 800px) {
    display: block;
  }
`;

/**
- A Drawer with links inside it
**/
export function Drawer({
  menuItens,
  as: Component,
  firstBrandLabel,
  secondBrandLabel,
  variant,
  openState,
  onOpen,
  onClose,
}) {
  const classes = useStyles();
  const [selectedIndex, setSelectedIndex] = useState(0);

  function Link({ to, children, click }) {
    if (click) {
      return (
        <a onClick={click} style={{ textDecoration: "none" }}>
          {children}
        </a>
      );
    }
    if (Component)
      return (
        <Component style={{ textDecoration: "none" }} to={to}>
          {children}
        </Component>
      );

    return (
      <a href={to} style={{ textDecoration: "none" }}>
        {children}
      </a>
    );
  }

  const handleListItemClick = (event, index) => {
    setSelectedIndex(index);
  };

  return (
    <>
      <div style={{ position: "absolute", left: 0, top: 0 }}>
        <StyledIconButton onClick={onOpen}>
          <MdMenu />
        </StyledIconButton>
      </div>
      <StyledMuiDrawer
        className={classes.drawer}
        variant={variant}
        open={openState}
        anchor="left"
        classes={{ paper: classes.drawerPaper }}
        onClose={onClose}
      >
        <List style={{ margin: "20px 0px" }}>
          <ListItem style={{ display: "flex", justifyContent: "space-around" }}>
            <Avatar
              variant="rounded"
              style={{ marginRight: "15px", backgroundColor: "#3B3B3A" }}
            >
              {firstBrandLabel.charAt(0)}
            </Avatar>
            <ListItemText style={{ color: "#3B3B3A" }}>
              <p
                style={{
                  fontSize: "16px",
                  fontWeight: 600,
                  margin: 0,
                  padding: 0,
                }}
                title={firstBrandLabel}
              >
                {firstBrandLabel.length > 10
                  ? firstBrandLabel.substring(0, 11).concat("...")
                  : firstBrandLabel}
              </p>
              <p
                style={{
                  color: "gray",
                  fontSize: "13px",
                  margin: 0,
                  padding: 0,
                }}
                title={secondBrandLabel}
              >
                {secondBrandLabel.length > 10
                  ? secondBrandLabel.substring(0, 20).concat("...")
                  : secondBrandLabel}
              </p>
            </ListItemText>
          </ListItem>
        </List>
        {menuItens.map(({ label, to, Icon, handleClick }, index) => (
          <Link key={index} to={to} click={handleClick}>
            <ListMenuItemStyled
              onClick={(event) => handleListItemClick(event, index)}
            >
              <ListItem
                button
                style={{ pointerEvents: "none" }}
                selected={selectedIndex === index}
              >
                <ListItemIconStyled>
                  {Icon ? <Icon style={{ fontSize: "24px" }} /> : <ListIcon />}
                </ListItemIconStyled>
                <ListItemText
                  primary={
                    <TypographyMenuItemStyled
                      type="body2"
                      style={{
                        fontFamily: typography.type.primary,
                      }}
                    >
                      {label}
                    </TypographyMenuItemStyled>
                  }
                />
              </ListItem>
            </ListMenuItemStyled>
          </Link>
        ))}
      </StyledMuiDrawer>
    </>
  );
}

Drawer.propTypes = {
  /**
   * Menu items to be displayed on button click.
   * Must follow the format: 
   * if u want to execute a function, pass a handleClick props
   * [{
        label: 'YourLabel',
        to: '/your/url/link',
        Icon: YourIcon,
        handleClick: YourFunction,
    },
    {
        label: 'YourLabel',
        to: '/your/url/link',
        Icon: YourIcon, 
    }]
   */
  menuItens: PropTypes.array,
  /**
   * The element that will be used to render the items. The component
   * must be able to receive a 'to' prop, which will be the redirect link
   * Example: <Link />
   */
  as: PropTypes.element,
  /**
   * The label to be your brandName
   */
  firstBrandLabel: PropTypes.string,
  /**
   * The second label to be your subtitle below brandName
   */
  secondBrandLabel: PropTypes.string,
  /**
   * The variant of drawer [permanent , persistent ,temporary]
   */
  variant: PropTypes.oneOf(["permanent", "persistent", "temporary"]),
  /**
   * The state to define if drawer menu is open or not
   * normally used when page is minor than 800 and hamburguer menu show
   * when use open state don't forget to use onOpen and onClose
   */
  openState: PropTypes.bool,
  /**
   * Function to change openState
   */
  onOpen: PropTypes.func,
  /**
   * Function to close openState
   */
  onClose: PropTypes.func,
};

Drawer.defaultProps = {
  menuItens: [],
  as: null,
  firstBrandLabel: "Your Brand Here",
  secondBrandLabel: "Your second Brand Here",
  variant: "permanent",
  openState: true,
  onOpen: () => null,
  onClose: () => null,
};
