import React, { useState, useRef } from "react";
import styled from "styled-components";
import PropTypes from "prop-types";

import { Button } from "../Button";

import { colors, typography } from "../../shared/styles";
import { useRefOutside } from "../../hooks/useRefOutsideClick";
import { MdArrowForward } from "react-icons/md";

const Container = styled.div`
  width: 150px;
  position: relative;
`;

const Dropdown = styled.ul`
  width: 95%;

  position: absolute;
  top: 10;

  background: #fff;

  display: flex;
  flex-direction: column;

  list-style: none;
  margin: 15px auto;
  padding: 0;
  border-radius: 8px;
  overflow: hidden;
  z-index: 999;

  font-family: ${typography.type.secondary};

  box-shadow: 0px 2px 4px rgba(40, 41, 61, 0.04),
    0px 8px 16px rgba(96, 97, 112, 0.16);
`;

const Anchor = styled.a`
  text-decoration: none;
  color: ${colors.darker};
`;

const MdArrowForwardStyled = styled(MdArrowForward)`
  color: #99ce86;
`;

const DropdownItem = styled.li`
  height: 35px;
  padding: 5px;
  font-weight: ${typography.weight.bold};

  display: flex;
  justify-content: space-between;
  align-items: center;

  cursor: pointer;

  text-decoration: none;

  :hover {
    background: ${colors.gossip};
    color: ${colors.lightest};
  }

  &:hover ${MdArrowForwardStyled} {
    color: white;
  }
`;

/**
- A dropdown button with links inside it
**/
export function DropdownButton({ children, as: Component, options, ...props }) {
  const wrapperRef = useRef(null);
  const [isOpen, setOpen] = useState(false);

  useRefOutside(wrapperRef, setOpen, false);

  function Link({ to, children }) {
    if (Component) return <Component to={to}>{children}</Component>;

    return <Anchor href={to}>{children}</Anchor>;
  }

  return (
    <Container ref={wrapperRef}>
      <Button {...props} fullWidth onClick={() => setOpen(!isOpen)}>
        {children}
      </Button>

      {isOpen && (
        <Dropdown style={{ margin: "3px" }}>
          {options.map(({ link, description }, index) => (
            <Link key={index} to={link}>
              <DropdownItem>
                {description} <MdArrowForwardStyled />
              </DropdownItem>
            </Link>
          ))}
        </Dropdown>
      )}
    </Container>
  );
}

DropdownButton.propTypes = {
  /**
   * The content that will be displayed inside the button
   */
  children: PropTypes.any,
  /**
   * The element that will be used to render the dropdown items. The component
   * must be able to receive a 'to' prop, which will be the redirect link
   * Example: <Link />
   */
  as: PropTypes.element,
  /**
   * Dropdown items to be displayed on button click.
   * Must follow the format: { description: 'Some item', link: '/some/link' }
   */
  options: PropTypes.arrayOf(
    PropTypes.shape({
      description: PropTypes.string.isRequired,
      link: PropTypes.string.isRequired,
    })
  ).isRequired,
};

DropdownButton.defaultProps = {
  children: "Items",
  as: null,
};
