import React from "react";
import PropTypes from "prop-types";

import {
  TextField,
  InputAdornment,
  styled,
  Avatar as AvatarMD,
  Grid,
  makeStyles,
} from "@material-ui/core";
import { MdInfoOutline } from "react-icons/md";

import NumberFormat from "react-number-format";

import { colors, typography } from "../../shared/styles";

const StyledTextField = styled(TextField)({
  "& label.Mui-focused": {
    color: colors.amulet,
  },
  "& .MuiInput-underline:after": {
    borderBottomColor: colors.amulet,
  },
  "& .MuiOutlinedInput-root": {
    "&:hover fieldset": {
      borderColor: colors.amulet,
    },
    "&.Mui-focused fieldset": {
      borderColor: colors.amulet,
    },
  },
});

const useStyles = makeStyles({
  required: {
    "& fieldset": {
      borderColor: colors.negative,
    }
  },
});

const CustomInput = ({ ...props }) => {
  const classes = useStyles();

  return props.isNumeric ? (
    <NumberFormat
      type="numeric"
      className={(props.required && !props.value) && classes.required}
      customInput={StyledTextField}
      format={props.mask}
      thousandSeparator="."
      decimalSeparator=","
      prefix={props.prefix}
      suffix={props.suffix}
      {...props}
    />
  ) : (
    <StyledTextField className={(props.required && !props.value) && classes.required} {...props} />
  );
};

/**
- Default input component
**/
export function Input({
  inputId,
  value,
  label,
  helperText,
  errorText,
  error,
  handleChange,
  fullWidth,
  textArea,
  minRows,
  maxRows,
  icon: Icon,
  size,
  placeholder,
  avatar,
  avatarName,
  isNumeric,
  type,
  mask,
  format,
  prefix,
  suffix,
  avatarTitle,
  ...props
}) {
  function createTextWithIcon() {
    return (
      <>
        {!error && helperText && (
          <>
            <MdInfoOutline style={{ fontSize: "13px" }} />
            {/* <img src={informationIcon} alt="InformationIcon" /> */}
            &nbsp;
          </>
        )}
        {error ? errorText : helperText}
      </>
    );
  }

  function handleSize() {
    if (size === "small") return "40px";
    if (size === "medium") return "50px";
  }

  return (
    <Grid container spacing={avatar ? 1 : 0} title={avatarTitle}>
      {avatar && (
        <Grid item lg={2}>
          <div
            style={{
              display: "flex",
              justifyContent: "center",
              alignItems: "center",
              height: "100%",
            }}
          >
            <AvatarMD
              style={{
                backgroundColor: colors.amulet,
                fontSize: `${typography.size.s1}px`,
              }}
            >
              {avatarName}
            </AvatarMD>
          </div>
        </Grid>
      )}

      <Grid item lg={avatar ? 10 : 12}>
        <CustomInput
          {...props}
          isNumeric={isNumeric}
          format={format}
          mask={mask}
          prefix={prefix}
          suffix={suffix}
          type={type}
          multiline={textArea}
          minRows={minRows}
          maxRows={maxRows}
          inputProps={{
            style: {
              padding: "0 14px 0 14px",
              height: handleSize(),
              fontFamily: typography.type.secondary,
              resize: textArea ? "vertical" : "none",
              fontSize: `${size === "small" ? typography.size.s1 : typography.size.s3
                }px`,
            },
          }}
          size={size}
          fullWidth={fullWidth}
          id={inputId}
          label={label}
          placeholder={placeholder}
          helperText={createTextWithIcon()}
          variant="outlined"
          error={error}
          value={value}
          name={inputId}
          onChange={(e) => handleChange(e)}
          InputProps={{
            endAdornment: Icon && (
              <InputAdornment position="end">
                <Icon />
              </InputAdornment>
            ),
          }}
        />
      </Grid>
    </Grid>
  );
}

Input.propTypes = {
  /**
   * The id provide to input
   */
  inputId: PropTypes.string,
  /**
   * The text to show in placeholder
   */
  label: PropTypes.string,
  /**
   * The text to show below the input
   */
  helperText: PropTypes.string,
  /**
   * The error text to show below the
   *  input in red
   */
  errorText: PropTypes.string,
  /**
   * Boolean to show errorText or not
   */
  error: PropTypes.bool,
  /**
   * The value of input
   */
  value: PropTypes.string.isRequired,
  /**
   * Function to handle with value change
   */
  handleChange: PropTypes.func.isRequired,
  /**
   * Boolean to show input fullWidth or not
   */
  fullWidth: PropTypes.bool,
  /**
   * Boolean to input transform in textarea
   */
  textArea: PropTypes.bool,
  /**
   * Minimum number of rows on input (usually used in textarea)
   */
  minRows: PropTypes.number,
  /**
   * Maximum number of rows on input (usually used in textarea)
   */
  maxRows: PropTypes.number,
  /**
   * Icon to be displayed inside the input
   */
  icon: PropTypes.object,
  /**
   * Input size, must be one of (small, medium). Default: medium
   */
  size: PropTypes.oneOf(["small", "medium"]),
  /**
   * Can be used to replace the label
   */
  placeholder: PropTypes.string,
  /**
   * Can be used to show a avatar before input
   */
  avatar: PropTypes.bool,
  /**
   * Can be used to show two letter in avatar
   */
  avatarName: PropTypes.string,
  /**
   * Can be used to transform the input in numeric
   */
  isNumeric: PropTypes.bool,
  /**
   * Input type
   */
  type: PropTypes.string,
  /**
   * _THIS FIELD WORKS JUST WHEN IS NUMERIC IS TRUE_  
   * Field Format. Example: (##) #####-####
   */
  format: PropTypes.string,
  /**
   * _THIS FIELD WORKS JUST WHEN IS NUMERIC IS TRUE_  
   * Field Mask to blank value mask = "."  will show: (..) .....-....
  */
  mask: PropTypes.string,
  /**
   * Prefix. Example: R$ 950,00 (R$)
   */
  prefix: PropTypes.string,
  /**
   * suffix. Example: 95% (%)
   */
  suffix: PropTypes.string,
  /**
   * title for an avatar description
   */
  avatarTitle: PropTypes.string,
};

Input.defaultProps = {
  inputId: "defaultId",
  label: null,
  helperText: "",
  errorText: "",
  error: false,
  fullWidth: false,
  textArea: false,
  minRows: 1,
  maxRows: 1,
  icon: null,
  size: "medium",
  placeholder: null,
  avatar: false,
  avatarName: "",
  isNumeric: false,
  type: "text",
  avatarTitle: "",
};
