import React, { useCallback } from "react";
import PropTypes from "prop-types";
import styled, { css } from "styled-components";

import { Pagination as MDPagination } from "@material-ui/lab";
import { NavigateNext, NavigateBefore } from "@material-ui/icons";

import { colors, typography } from "../../shared/styles";

function getItemColor(fillBackground, selected) {
  if (fillBackground) return colors.lightest;
  if (selected) return colors.amulet;

  return colors.mediumdark;
}

const PageNumber = styled.button`
  border-style:none;
  background-color: transparent;

  width: 20px;
  height: 20px;
  border-radius: 50%;

  padding: 5px;

  display: flex;
  justify-content: center;
  align-items: center;

  font-size: ${typography.size.s1}px;
  font-family: ${typography.type.secondary};
  font-weight: ${typography.weight.bold};
  cursor: pointer;

  color: ${({ selected, fillBackground }) =>
    getItemColor(fillBackground, selected)};

  ${({ fillBackground }) =>
    fillBackground &&
    css`
      background: ${colors.gossip};
      box-shadow: 0px 0px 2px rgba(40, 41, 61, 0.04),
        0px 4px 8px rgba(96, 97, 112, 0.16);
    `}

  ${({ last }) =>
    last &&
    css`
      margin-left: 20px;
    `}

  ${({ first }) =>
    first &&
    css`
      margin-right: 20px;
    `}
`;

function PaginationItem({ page, type, selected, onClick, disabled }) {
  if (type === "previous")
    return (
      <PageNumber onClick={onClick} first disabled={disabled}>
        <NavigateBefore />
      </PageNumber>
    );

  if (type === "next")
    return (
      <PageNumber onClick={onClick} last disabled={disabled}>
        <NavigateNext />
      </PageNumber>
    );

  if (type === "end-ellipsis")
    return <PageNumber onClick={onClick} disabled={disabled}>{"..."}</PageNumber>;

  return (
    <PageNumber onClick={onClick} selected={selected} disabled={disabled}>
      {page}
    </PageNumber>
  );
}

/**
  Default pagination component
**/
export function Pagination({ page, pages, paginate }) {
  const handlePaginate = useCallback(
    (_, value) => {
      if (value !== page) paginate(value);
    },
    [paginate, page]
  );

  const isDisabled = pages === 0 || pages === 1;

  return (
    <MDPagination
      disabled={isDisabled}
      count={pages}
      page={page}
      onChange={handlePaginate}
      renderItem={(props) => <PaginationItem {...props} />}
    />
  );
}

Pagination.propTypes = {
  /**
   * The actual page
   */
  page: PropTypes.number.isRequired,
  /**
   * The number of pages
   */
  pages: PropTypes.number.isRequired,
  /**
   * Function to handle pagination. It must receive a page parameter
   * that represents the selected page
   */
  paginate: PropTypes.func.isRequired,
};
