import React from "react";
import PropTypes from "prop-types";
import {
  FormControl,
  FormHelperText,
  InputLabel,
  makeStyles,
  Select as MDSelect,
  styled,
} from "@material-ui/core";

import { colors, typography } from "../../shared/styles";

const CssFormControl = styled(FormControl)({
  "& label.Mui-focused": {
    color: colors.amulet,
  },
  "& .MuiInput-underline:after": {
    borderBottomColor: colors.amulet,
  },
  "& .MuiOutlinedInput-root": {
    "&:hover fieldset": {
      borderColor: colors.amulet,
    },
    "&.Mui-focused fieldset": {
      borderColor: colors.amulet,
    },
  },
});

const useStyles = makeStyles({
  required: {
    "& fieldset": {
      borderColor: colors.negative,
    },
  },
});

const CustomCssFormControl = ({...props }) => {
  const classes = useStyles();

  return <CssFormControl className={(props.required && !props.value) && classes.required} {...props} />;
};

export function Select({
  selectId,
  label,
  value,
  error,
  errorText,
  handleChange,
  options,
  size,
  ...props
}) {
  function handleSize() {
    if (size === "small") return "40px";
    if (size === "medium") return "50px";
  }
  return (
    <div>
      <CustomCssFormControl
        {...props}
        variant="outlined"
        fullWidth
        hiddenLabel
        error={error}
        size={size}
        value={value}
      >
        {label && <InputLabel>{label}</InputLabel>}
        <MDSelect
          style={{ fontFamily: typography.type.primary, height: handleSize() }}
          key={selectId}
          name={selectId}
          native
          error={error}
          value={value}
          onChange={(e) => handleChange(e)}
          label={label}
          {...props}
        >
          {options.map((e) => (
            <option disabled={e.disabled} value={e.value} key={e.value}>
              {e.label}
            </option>
          ))}
        </MDSelect>
        {error && <FormHelperText>{errorText}</FormHelperText>}
      </CustomCssFormControl>
    </div>
  );
}

// Important information, this component is only fullwidth according to its parent.
Select.propTypes = {
  /**
   * The id provide to select
   */
  selectId: PropTypes.string,
  /**
   * The text to show in placeholder
   */
  label: PropTypes.string,
  /**
   * Boolean to show error color on select
   */
  error: PropTypes.bool,
  /**
   * Error message to be displayed
   */
  errorText: PropTypes.string,
  /**
   * The value of select
   */
  value: PropTypes.number,
  /**
   * Function to handle with value change
   */
  handleChange: PropTypes.func,
  /**
   * Option values to show on select [{value: value, label: label}]
   */
  options: PropTypes.array,
  /**
   * The size of the select.
   * Accepted sizes: 'large', 'medium', 'small'
   */
  size: PropTypes.oneOf(["medium", "small"]),
};

Select.defaultProps = {
  selectId: "defaultId",
  label: null,
  error: false,
  value: 0,
  handleChange: null,
  options: [],
  size: "medium",
};
