import React, { useState, useMemo, useCallback, useEffect } from "react";
import PropTypes from "prop-types";

import {
  Table as MuiTable,
  TableBody as MuiTableBody,
  TableCell as MuiTableCell,
  TableHead as MuiTableHead,
  TableRow as MuiTableRow,
} from "@material-ui/core";
import { Search } from "@material-ui/icons";

import { Pagination } from "../Pagination";
import { Input } from "../Input";

import styled, { css } from "styled-components";

import { typography, colors } from "../../shared/styles";

/**
 * Table header cell component
 * @param {object} props
 * @returns HeaderCell
 */
function HeaderCell({ column, center, ordering, onChangeOrder }) {
  const orderAvailable = useMemo(() => {
    return ordering?.availableOrderBy?.some((o) => o === column.id) || false;
  }, [ordering, column]);

  const props = useMemo(() => {
    const ordered = ordering?.order_by === column.id;
    const order = ordered ? ordering?.order : null;

    return {
      sorting: orderAvailable,
      [order]: true,
    };
  }, [orderAvailable, ordering, column]);

  const onOrder = useCallback(() => {
    if (!orderAvailable) return;

    const getOrder = () => {
      if (column.id === ordering?.order_by) {
        const isAsc = ordering?.order === "asc";
        return isAsc ? "desc" : "asc";
      }

      return "asc";
    };

    const order = getOrder();

    onChangeOrder({ order_by: column.id, order });
  }, [onChangeOrder, column, ordering, orderAvailable]);

  return (
    <TableCell
      key={column.id}
      center={center}
      textBold
      onClick={onOrder}
      {...props}>
      <p
        style={{
          margin: "0px",
          color: "#8A8A8A",
          fontWeight: 600,
          fontSize: "14px",
          lineHeight: "22px",
        }}>
        {column.title}
      </p>
    </TableCell>
  );
}

HeaderCell.propTypes = {
  /**
   * Table columns. Must have an id and a title. It also can be a react
   * component
   */
  column: PropTypes.shape({
    id: PropTypes.string.isRequired,
    title: PropTypes.string.isRequired,
    component: PropTypes.func,
  }).isRequired,
  /**
   * Center the text inside the cell
   */
  center: PropTypes.bool,
  /**
   * Table ordering data
   */
  ordering: PropTypes.shape({
    /** current column to order by */
    order_by: PropTypes.string,
    /** asc | desc  */
    order: PropTypes.string,
    /** array of available columns to order by */
    availableOrderBy: PropTypes.arrayOf(PropTypes.string),
  }),
  /**
   * Function to handle order change
   */
  onChangeOrder: PropTypes.func,
};

const TableCell = styled(MuiTableCell)`
  font-family: ${typography.type.secondary} !important;
  font-size: ${typography.size.s2}px !important;
  border-bottom: 0 !important;

  ${({ textBold }) =>
    textBold &&
    css`
      font-weight: ${typography.weight.bold} !important;
    `}

  ${({ center }) =>
    center &&
    css`
      text-align: center !important;
    `}

  ${({ sorting, asc, desc }) =>
    sorting &&
    css`
      position: relative;
      padding-right: 30px !important;
      cursor: pointer;

      ::before {
        content: "\\2191";
        position: absolute;
        right: 1em;
        bottom: 18px;
        opacity: 0.3;
        display: block;
      }

      ::after {
        content: "\\2193";
        position: absolute;
        right: 0.5em;
        bottom: 18px;
        opacity: 0.3;
        display: block;
      }

      ${asc &&
      css`
        ::before {
          opacity: 1;
        }
      `}

      ${desc &&
      css`
        ::after {
          opacity: 1;
        }
      `}
    `}
`;

const Container = styled.div`
  height: 64px;
  padding: 8px 16px;

  display: flex;
  justify-content: space-between;
  align-items: center;

  box-shadow: 0px 0px 1px rgba(40, 41, 61, 0.08),
    0px 0.5px 2px rgba(96, 97, 112, 0.16);
`;

const PaginationInfo = styled.p`
  font-family: ${typography.type.secondary};
  font-size: ${typography.size.s2}px;

  color: ${colors.darker};
`;

const TableContainer = styled.div`
  background: ${colors.lightest};
`;

/**
 * Table component
 * @param {object} props
 * @returns Table
 */
export function Table({
  columns,
  rows,
  paging,
  showPaging,
  onChangePage,
  ordering,
  onChangeOrder,
  showSearch,
  onSearch,
  searchPlaceholder,
  refValue,
  headerComponent: HeaderComponent,
  headerTitle,
  formatInputProps,
  maskInputProps,
  isNumericInputProps,
}) {
  const [ref, setRef] = useState("");

  useEffect(() => {
    setRef(refValue);
  }, [refValue]);

  const handleSubmit = useCallback(
    (e) => {
      e?.preventDefault();

      onSearch(ref);
      // setRef('');
    },
    [onSearch, ref]
  );

  const handlePaginateDescription = useCallback((limit, page, total) => {
    const numberOfRegisters = (limit * page) > total ? total : (limit * page);
    const totalOfRegisters = total;

    return (
      <>
        Exibindo <b>{numberOfRegisters}</b> de <b>{totalOfRegisters}</b>
      </>
    )
  }, [])

  return (
    <TableContainer>
      {headerTitle && (
        <div
          style={{
            display: "flex",
            alignContent: "center",
          }}>
          <p
            style={{
              fontFamily: typography.type.secondary,
              color: "#3B3B3A",
              alignContent: "center",
              height: "600",
              fontSize: "24px",
              lineHeight: "32px",
              margin: "16px",
              fontWeight: 600,
            }}>
            {headerTitle}
          </p>
        </div>
      )}

      <Container>
        {showSearch && (
          <div style={{ width: 500 }}>
            <form onSubmit={handleSubmit}>
              <Input
                value={ref}
                handleChange={(e) => setRef(e.target.value)}
                placeholder={searchPlaceholder}
                icon={() => (
                  <Search
                    style={{ cursor: "pointer" }}
                    htmlColor="#8A8A8A"
                    onClick={handleSubmit}
                  />
                )}
                size="small"
                fullWidth
                format={formatInputProps}
                mask={maskInputProps}
                isNumeric={isNumericInputProps}
              />
            </form>
          </div>
        )}
        {HeaderComponent && <HeaderComponent />}
      </Container>

      <MuiTable size="medium">
        <MuiTableHead>
          <MuiTableRow>
            {columns.map((column, index) => (
              <HeaderCell
                key={column.id}
                column={column}
                center={index !== 0}
                ordering={ordering}
                onChangeOrder={onChangeOrder}
              />
            ))}
          </MuiTableRow>
        </MuiTableHead>

        <MuiTableBody>
          {rows.map((item, index) => (
            <MuiTableRow
              style={{
                borderBottom: "1px solid #E0E0E0",
              }}
              key={item.id || index}>
              {columns.map((column, colIndex) => {
                if (column.component) {
                  const { component: Component } = column;

                  return (
                    <Component key={column.id} item={item} column={column} />
                  );
                }

                return (
                  <>
                    <TableCell center={colIndex !== 0} key={column.id}>
                      <p
                        style={{
                          color: "#3B3B3A",
                          padding: "15px 15px",
                        }}>
                        {item[column.id] || "-"}
                      </p>
                    </TableCell>
                  </>
                );
              })}
            </MuiTableRow>
          ))}
        </MuiTableBody>
      </MuiTable>

      {showPaging && (
        <Container>
          <Pagination
            page={paging?.page}
            pages={Math.ceil(paging?.total / paging?.limit)}
            paginate={onChangePage}
          />

          <PaginationInfo style={{ fontWeight: 400, color: "#8A8A8A" }}>
            {/* Exibindo <b>{paging?.limit * paging?.page}</b> de{" "}
            <b>{paging?.total}</b> */}
            {handlePaginateDescription(paging?.limit, paging?.page, paging?.total)}
          </PaginationInfo>
        </Container>
      )}
    </TableContainer>
  );
}

Table.propTypes = {
  /**
   * An array of rows to be displayed, each row must have an id property
   * which will be used to relate to its column.
   * Example:
   * [
   *  { name: 'Frozen yoghurt', calories: 159, fat: 6.0, carbs: 24, protein: 4.0 },
   *  { name: 'Ice cream sandwich', calories: 237, fat: 9.0, carbs: 37, protein: 4.3 },
   *  { name: 'Eclair', calories: 262, fat: 16.0, carbs: 24, protein: 6.0 },
   *  { name: 'Cupcake', calories: 305, fat: 3.7, carbs: 67, protein: 4.3 },
   *  { name: 'Gingerbread', calories: 356, fat: 16.0, carbs: 49, protein: 3.9 },
   * ];
   */
  rows: PropTypes.arrayOf(
    PropTypes.shape({
      id: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
    })
  ).isRequired,
  /**
   * Table columns. Must have an id and a title. It also can be a react
   * component.
   * Example:
   * [
   *  { id: 'name', title: 'Dessert (100g serving)' },
   *  { id: 'calories', title: 'Calories' },
   *  { id: 'fat', title: 'Fat (g)' },
   *  { id: 'carbs', title: 'Carbs (g)' },
   *  { id: 'protein', title: 'Protein (g)' },
   * ]
   */
  columns: PropTypes.arrayOf(
    PropTypes.shape({
      id: PropTypes.string.isRequired,
      title: PropTypes.string.isRequired,
      component: PropTypes.func,
    })
  ).isRequired,

  /**
   * Paging data
   */
  paging: PropTypes.shape({
    limit: PropTypes.number,
    page: PropTypes.number,
    /** total items */
    total: PropTypes.number,
  }),
  /**
   * Function to handle page change, it must receive a page parameter
   */
  onChangePage: PropTypes.func,
  /**
   * If true, paging component will be displayed. Default: true
   */
  showPaging: PropTypes.bool,

  /**
   * Table ordering data
   */
  ordering: PropTypes.shape({
    /** current column to order by */
    order_by: PropTypes.string,
    /** asc | desc  */
    order: PropTypes.string,
    /** array of available columns to order by */
    availableOrderBy: PropTypes.arrayOf(PropTypes.string),
  }),
  /**
   * Function to handle order change, it must be able to receive a ordering
   * like the example:
   * { order_by: 'col', order: 'asc' }
   */
  onChangeOrder: PropTypes.func,
  /**
   * If true, the search container will be displayed. Default: true
   */
  showSearch: PropTypes.bool,
  /**
   * Component to be displayed on table top right position
   */
  headerComponent: PropTypes.func,
  /**
   * Function to handle search form submit. Must be able to receive a single
   * param which is the value of the search
   */
  onSearch: PropTypes.func,
  /**
   * Placeholder to search input. Default: Pesquisar
   */
  searchPlaceholder: PropTypes.string,
  /**
   * Value for input start with something on search
   */
  refValue: PropTypes.string,
  /**
   * Value for title on the top of table
   */
  headerTitle: PropTypes.string,
  /**
   * Can be used to transform the input in numeric
   */
  isNumericInputProps: PropTypes.bool,
  // IMPORTANT NOTE: TO APPLY MASK AND FORMAT U NEED TO SET INPUT ISNUMERIC
  /**
   * Field Mask to blank value mask = '_' will show: _ _ _ . _ _ _ . _ _ _ - _ _
   */
  maskInputProps: PropTypes.string,
  /**
   * Field Format. Example: (##) #####-####
   */
  formatInputProps: PropTypes.string,
};

Table.defaultProps = {
  showPaging: true,
  showSearch: true,
  searchPlaceholder: "Pesquisar",
  refValue: "",
};
