import React, { useRef, useState, useCallback } from 'react';
import PropTypes from 'prop-types';

import { Editor, EditorState, RichUtils, getDefaultKeyBinding } from 'draft-js';
import { convertFromHTML, convertToHTML } from 'draft-convert';

import { InlineControls } from './InlineControls';
import { Container, Header, TextEditorContainer } from './styles';
import { contains } from '../../utils/arrays';

const allowedCommands = [
  'backspace',
  'backspace-word',
  'split-block',
  'bold',
  'italic'
];

/**
  Text editor component
  It returns a HTML formatted text
**/
export function TextEditor({ initialValue, onChange }) {
  const ref = useRef(null);

  const [editorState, setEditorState] = useState(
    initialValue
      ? () => EditorState.createWithContent(convertFromHTML(initialValue))
      : () => EditorState.createEmpty()
  );

  const handleSetState = useCallback((newState) => {
    setEditorState(newState);

    onChange(convertToHTML(newState.getCurrentContent()));
  }, [onChange]);

  const handleKeyCommand = useCallback((command, state) => {
    if (!contains(allowedCommands, command)) return;

    const newState = RichUtils.handleKeyCommand(state, command);

    if (newState) {
      handleSetState(newState);
      return 'handled';
    }

    return 'not-handled';
  }, [handleSetState]);

  const toggleInlineStyle = useCallback((style) => {
    const newState = RichUtils.toggleInlineStyle(editorState, style);

    handleSetState(newState);
  }, [editorState, handleSetState]);

  const mapKeyToEditorCommand = useCallback((e) => {
    const TAB_CODE = 9;
    const MAX_DEPTH = 4;

    if (e.keyCode === TAB_CODE) {
      const newState = RichUtils.onTab(e, editorState, MAX_DEPTH);

      if (newState !== editorState) handleSetState(newState);

      return;
    }

    return getDefaultKeyBinding(e);
  }, [editorState, handleSetState]);

  const focus = useCallback(() => ref.current?.focus(), []);

  return (
    <Container>
      <Header>
        <InlineControls
          editorState={editorState}
          onToggle={toggleInlineStyle}
        />
      </Header>

      <TextEditorContainer onClick={focus}>
        <Editor
          editorState={editorState}
          handleKeyCommand={handleKeyCommand}
          onChange={handleSetState}
          keyBindingFn={mapKeyToEditorCommand}
          ref={ref}
          spellCheck
        />
      </TextEditorContainer>
    </Container>
  );
}

TextEditor.propTypes = {
  /**
   * An HTML formatted string value
   */
  initialValue: PropTypes.string,
  /**
   * On change event to be called with a single param, representing
   * the HTML current value of the editor
   */
  onChange: PropTypes.func.isRequired,
};

TextEditor.defaultProps = {
  initialValue: null,
};
